@@ -1,6 +1,6 @@
 ;;; haskell-font-lock.el --- Font locking module for Haskell Mode
 
-;; Copyright 2003, 2004  Free Software Foundation, Inc.
+;; Copyright 2003, 2004, 2005  Free Software Foundation, Inc.
 ;; Copyright 1997-1998 Graeme E Moss, and Tommy Thorn
 
 ;; Authors: 1997-1998 Graeme E Moss <gem@cs.york.ac.uk> and
@@ -8,7 +8,7 @@
 ;;          2003  Dave Love <fx@gnu.org>
 ;; Keywords: faces files Haskell
 
-;;; This file is not part of GNU Emacs.
+;; This file is not part of GNU Emacs.
 
 ;; This file is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -94,8 +94,8 @@
 ;; . Support for GreenCard?
 ;;
 
-;;; All functions/variables start with
-;;; `(turn-(on/off)-)haskell-font-lock' or `haskell-fl-'.
+;; All functions/variables start with
+;; `(turn-(on/off)-)haskell-font-lock' or `haskell-fl-'.
 
 ;;; Code:
 
@@ -105,7 +105,7 @@
 (require 'font-lock)
 
 ;; Version.
-(defconst haskell-font-lock-version "1.12"
+(defconst haskell-font-lock-version "1.17"
   "Version number of haskell-font-lock.")
 (defun haskell-font-lock-version ()
   "Echo the current version of haskell-font-lock in the minibuffer."
@@ -127,20 +127,29 @@ and `unicode'."
 
 (defconst haskell-font-lock-symbols-alist
   (append
+   ;; Prefer single-width Unicode font for lambda.
+   (and (fboundp 'decode-char)
+	(memq haskell-font-lock-symbols '(t unicode))
+	(list (cons "\\" (decode-char 'ucs 955))))
    ;; The symbols can come from a JIS0208 font.
    (and (fboundp 'make-char) (charsetp 'japanese-jisx0208)
 	(memq haskell-font-lock-symbols '(t japanese-jisx0208))
-	(list (cons "\\" (make-char 'japanese-jisx0208 38 75))
+	(list (cons "not" (make-char 'japanese-jisx0208 34 76))
+	      (cons "\\" (make-char 'japanese-jisx0208 38 75))
 	      (cons "->" (make-char 'japanese-jisx0208 34 42))
 	      (cons "<-" (make-char 'japanese-jisx0208 34 43))
 	      (cons "=>" (make-char 'japanese-jisx0208 34 77))))
    ;; Or a unicode font.
    (and (fboundp 'decode-char)
 	(memq haskell-font-lock-symbols '(t unicode))
-	(list (cons "\\" (decode-char 'ucs 955))
-	      (cons "->" (decode-char 'ucs 8594))
+	(list (cons "not" (decode-char 'ucs 172))
+              (cons "->" (decode-char 'ucs 8594))
 	      (cons "<-" (decode-char 'ucs 8592))
 	      (cons "=>" (decode-char 'ucs 8658))
+              (cons "~>" (decode-char 'ucs 8669)) ;; Omega language
+              ;; (cons "~>" (decode-char 'ucs 8605)) ;; less desirable
+              (cons "-<" (decode-char 'ucs 8610)) ;; Paterson's arrow syntax
+              ;; (cons "-<" (decode-char 'ucs 10521)) ;; nicer but uncommon
 	      (cons "::" (decode-char 'ucs 8759))
 	      (cons "." (decode-char 'ucs 9675))))))
 
@@ -169,10 +178,16 @@ Assume this means we have other useful features from Emacs 21.")
   "Compose a sequence of ascii chars into a symbol.
 Regexp match data 0 points to the chars."
   ;; Check that the chars should really be composed into a symbol.
-  (let ((start (match-beginning 0))
-	(end (match-end 0)))
-    (if (or (memq (char-syntax (or (char-before start) ?\ )) '(?_ ?\\))
-	    (memq (char-syntax (or (char-after end) ?\ )) '(?_ ?\\))
+  (let* ((start (match-beginning 0))
+         (end (match-end 0))
+	 (syntaxes (cond
+                    ((eq (char-syntax (char-after start)) ?w) '(?w))
+                    ;; Special case for the . used for qualified names.
+                    ((and (eq (char-after start) ?\.) (= end (1+ start)))
+                     '(?_ ?\\ ?w))
+                    (t '(?_ ?\\)))))
+    (if (or (memq (char-syntax (or (char-before start) ?\ )) syntaxes)
+	    (memq (char-syntax (or (char-after end) ?\ )) syntaxes)
 	    (memq (get-text-property start 'face)
 		  '(font-lock-doc-face font-lock-string-face
 		    font-lock-comment-face)))
@@ -287,6 +302,13 @@ Returns keywords suitable for `font-lock-keywords'."
 		;; Expensive.
 		`((,string-and-char 1 font-lock-string-face)))
 
+            ;; This was originally at the very end (and needs to be after
+            ;; all the comment/string/doc highlighting) but it seemed to
+            ;; trigger a bug in Emacs-21.3 which caused the compositions to
+            ;; be "randomly" dropped.  Moving it earlier seemed to reduce
+            ;; the occurrence of the bug.
+	    ,@(haskell-font-lock-symbols-keywords)
+
 	    (,reservedid 1 (symbol-value 'haskell-keyword-face))
 	    (,reservedsym 1 (symbol-value 'haskell-operator-face))
 
@@ -310,8 +332,7 @@ Returns keywords suitable for `font-lock-keywords'."
 	    ;; Very expensive.
 	    (,sym 0 (if (eq (char-after (match-beginning 0)) ?:)
 			haskell-constructor-face
-		      haskell-operator-face))
-	    ,@(haskell-font-lock-symbols-keywords)))
+		      haskell-operator-face))))
     (unless haskell-emacs21-features
       (case literate
 	(bird
@@ -387,8 +408,9 @@ that should be commented under LaTeX-style literate scripts."
     ;; ("^[ \t]*\\(\\\\\\)" (1 "."))
     ;; Deal with instances of `--' which don't form a comment.
     ("\\s_\\{3,\\}" (0 (if (string-match "\\`-*\\'" (match-string 0))
-			   nil		; Sequence of hyphens.  Do nothing in
-					; case of things like `{---'.
+                           ;; Sequence of hyphens.  Do nothing in
+                           ;; case of things like `{---'.
+			   nil
 			 "_")))))	; other symbol sequence
 
 (defconst haskell-bird-syntactic-keywords
@@ -522,8 +544,9 @@ Use `haskell-font-lock-version' to find out what version this is."
   (interactive)
   (font-lock-mode -1))
 
-;;; Provide ourselves:
+;; Provide ourselves:
 
 (provide 'haskell-font-lock)
 
+;; arch-tag: 89fd122e-8378-4c7f-83a3-1f49a64e458d
 ;;; haskell-font-lock.el ends here
